/* $Id: frecv.cpp,v 1.8 2004/12/15 10:08:56 roca Exp $ */
/*
 *  Copyright (c) 2003-2004 INRIA - All rights reserved
 *  main authors: Christoph Neumann - christoph.neumann@inrialpes.fr
 *		  Vincent Roca - vincent.roca@inrialpes.fr
 *		  Julien Laboure - julien.laboure@inrialpes.fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */


/*
 * frecv.c
 *
 *	flute receiver side functions
 */
#include "flute.h"


void FluteRecv(void)
{
	int	Bytes_received = 0;
	int	mcl_option;

	if (!interactive) {
		mcl_option = 1;
	} else {
		mcl_option = 0;
	}
	if (mcl_ctl(id, MCL_OPT_SET_FLUTE_STORE_ALL_ADUS_BY_DEFAULT,
		    (void*)&mcl_option, sizeof(mcl_option))) {
		EXIT(("mcl_ctl: MCL_OPT_SET_FLUTE_STORE_ALL_ADUS_BY_DEFAULT failed\n"))
	}

	if (optimode == OPTIMIZE_SPACE) {
		mcl_option = 0;
		if (mcl_ctl(id, MCL_OPT_POSTPONE_FEC_DECODING,
				(void*)&mcl_option, sizeof(mcl_option))) {
			/* non critical... ignore! */
			//EXIT(("mcl_ctl: MCL_OPT_POSTPONE_FEC_DECODING failed\n"))
		}

	} else if (optimode == OPTIMIZE_SPEED) {
		mcl_option = 0;
		if (mcl_ctl(id, MCL_OPT_POSTPONE_FEC_DECODING,
				(void*)&mcl_option, sizeof(mcl_option))) {
			/* non critical... ignore! */
			//EXIT(("mcl_ctl: MCL_OPT_POSTPONE_FEC_DECODING failed\n"))
		}
	} else if (optimode == OPTIMIZE_CPU) {
		mcl_option = 1;
		if (mcl_ctl(id, MCL_OPT_POSTPONE_FEC_DECODING,
				(void*)&mcl_option, sizeof(mcl_option))) {
			/*
			 * non critical... ignore!
			 * for instance, this is only valid if RSE is used,
			 * but does not apply to LDPC/LDGM
			 */
			//EXIT(("mcl_ctl: MCL_OPT_POSTPONE_FEC_DECODING failed\n"))
		}
	} else {
		EXIT(("FATAL ERROR: invalid optimization mode!"))
	}

	if (src_addr > 0) {
		/* in host format! */
		if (mcl_ctl(id, MCL_OPT_SRC_ADDR, (void*)&src_addr,
				sizeof(src_addr)))
			EXIT(("mcl_ctl: MCL_OPT_SRC_ADDR failed\n"))
	}

	PRINT(("Waiting for data...\n"))
	Bytes_received = RecvFiles();

	mcl_close(id);
	PRINT(("\nFluteRecv completed, %d bytes received\n", Bytes_received))
}


int RecvFiles (void)
{
	char		*buf_file = NULL;	/* buffer for recv'd fragment */
	int		len = 0;		/* available object length */
						/* returned from GET_INFO */
	int		len2 = 0;		/* actual object length after */
						/* mcl_recvmsg() call */
	int		fdt_len;		/* actual FDT instance length */
	pFFile		ThisFile = NULL;
	unsigned int 	toi;
	int		received = 0;		/* total amount of data recv'd*/
	struct mcl_msghdr	mh;		/* for mcl_recvsmg */
	struct mcl_iovec	iov;		/* for mcl_recvsmg */

	/*
	 * Receive ALL objects...
	 * Here an object can be either a file or an FDT
	 */
	memset((void*)&mh, 0, sizeof(mh));
	while ((len = mcl_recvmsg(id, &mh,
			  MCL_MSG_GET_INFO_FOR_AVAILABLE_OBJECT)) != -1) {
		flute_lock(&flutemutex);
		toi = (UINT32)mh.toi;
#ifdef DEBUG
		PRINT(("Flute: New Object Received: %d Bytes, TOI=%u\n", len, toi))
#endif
		if (toi == 0) {
			/*
			 * this is an FDT Instance, so receive the object
			 * in a buffer
			 */
			if (!(buf_file = (char*)malloc(len))) {
				EXIT(("Flute: Error: Cannot alloc memory!\n"))
			}
			/* a simple mcl_recv is sufficient here... */
			fdt_len = mcl_recv(id, buf_file, len);
			if (fdt_len < 0 || fdt_len != len) {
				EXIT(("Flute: Error, mcl_recv failed for FDT, fdt_len=%d\n", fdt_len))
			}
			updateFDT(buf_file, len);
#ifdef WIN32
			displayFDT(0, myfiles);
#else
			if (verbose != 0)
				displayFDT(0, myfiles);
#endif
			if (!interactive)
				myfiles = updateFFile(myfiles);
			/* free this buffer, since next object may be
			 * significantly shorter or larger */
			free(buf_file);
			buf_file = NULL;		

		} else if ((ThisFile = FFileFindTOI(toi, myfiles)) != NULL) {
			/*
			 * this is a known and selected file
			 */
			/*
			 * Content-Length Check, if this is specified in
			 * an FDT Instance (i.e. if filesize != 0)
			 */
			if ((ThisFile->filesize != 0) && (len != ThisFile->filesize)) {
				EXIT(("Flute: Error, actual length (%d) different from length specified in FDT (%ld) for TOI %u\n", len, ThisFile->filesize, toi))
			}
			if (ThisFile->writeIt) {
				/*
				 * this file can be written/over-written, so
				 * prepar the recvmsg call
				 */
				memset((void*)&mh, 0, sizeof(mh));
				mh.msg_iov = &iov;
				mh.msg_iovlen = 1;
				iov.iov_type = MCL_IOV_TYPE_FILE;
				iov.iov_base = NULL;
				iov.iov_filename = ThisFile->fullname;
				iov.iov_offset = (INT64)0;
				iov.iov_len = (INT64)len;
				len2 = mcl_recvmsg(id, &mh, MCL_MSG_DEFAULT);
				if (len2 != len) {
					EXIT(("Flute: Error, mcl_recvmsg failed, returned %d\n", len))
				}
				PRINT(("Flute: Received file \"%s\" (%d Bytes)\n",
					ThisFile->fullname, len))
				ThisFile->received = 1;
#ifdef WIN32
				received += len;
#else
				/*MD5sum checks*/
				if (strcmp((const char *)ThisFile->md5sum,"\0")==0) {
					PRINT(("Flute: No md5sum information for file %s\n", ThisFile->fullname));
					ThisFile->integrity = 0;
					received += len;
				} else if (!FFilemd5sum_check(ThisFile)) {
					EXIT(("Flute: WARNING: wrong md5sum for file %s\n", ThisFile->fullname));
					//ThisFile->integrity=-1;
				} else {
					PRINT(("Flute: MD5sum OK, for file %s\n", ThisFile->fullname))
					ThisFile->integrity=1;			
					received += len;
				}
#endif
			} else {
				PRINT(("Flute: Skipped file \"%s\"\n", ThisFile->fullname))
			}
		} else {
			/*
			 * this is an unknown or unwanted file
			 */
			PRINT(("Flute: WARNING: Received unknown object TOI %u\n",
				toi))
		}
		flute_unlock(&flutemutex);
		/* reset for next mcl_recvmsg */
		memset((void*)&mh, 0, sizeof(mh));
	}
	return received;
}

